# Implementation Plan: Rhai Wrappers for Text Tools

## 1. Overview

We'll create a new module `rhai/text.rs` that will provide Rhai wrappers for all functionality in the text module, including:
- TextReplacer (from replace.rs)
- TemplateBuilder (from template.rs)
- name_fix and path_fix functions (from fix.rs)
- dedent and prefix functions (from dedent.rs)

The implementation will follow the builder pattern for the TextReplacer and TemplateBuilder classes, similar to their Rust implementations, and will use the same error handling pattern as the existing Rhai modules.

## 2. Module Structure

```mermaid
graph TD
    A[rhai/mod.rs] --> B[rhai/text.rs]
    B --> C[TextReplacer Wrappers]
    B --> D[TemplateBuilder Wrappers]
    B --> E[Fix Function Wrappers]
    B --> F[Dedent Function Wrappers]
    B --> G[Error Handling]
    B --> H[Registration Functions]
```

## 3. Implementation Details

### 3.1. Module Setup and Registration

1. Create a new file `rhai/text.rs`
2. Add the module to `rhai/mod.rs`
3. Implement a `register_text_module` function to register all text-related functions with the Rhai engine
4. Update the main `register` function in `rhai/mod.rs` to call `register_text_module`

### 3.2. TextReplacer Implementation

1. Register the TextReplacer type with the Rhai engine
2. Implement the following functions:
   - `text_replacer_new()` - Creates a new TextReplacerBuilder
   - `pattern(builder, pat)` - Sets the pattern to search for and automatically adds any previous pattern/replacement pair to the chain
   - `replacement(builder, rep)` - Sets the replacement text
   - `regex(builder, yes)` - Sets whether to use regex
   - `case_insensitive(builder, yes)` - Sets whether the replacement should be case-insensitive
   - `build(builder)` - Builds the TextReplacer with all configured replacement operations
   - `replace(replacer, input)` - Applies all configured replacement operations to the input text
   - `replace_file(replacer, path)` - Reads a file, applies all replacements, and returns the result as a string
   - `replace_file_in_place(replacer, path)` - Reads a file, applies all replacements, and writes the result back to the file
   - `replace_file_to(replacer, input_path, output_path)` - Reads a file, applies all replacements, and writes the result to a new file

### 3.3. TemplateBuilder Implementation

1. Register the TemplateBuilder type with the Rhai engine
2. Implement the following functions:
   - `template_builder_open(template_path)` - Creates a new TemplateBuilder with the specified template path
   - `add_var(builder, name, value)` - Adds a variable to the template context
   - `add_vars(builder, vars_map)` - Adds multiple variables to the template context from a Map
   - `render(builder)` - Renders the template with the current context
   - `render_to_file(builder, output_path)` - Renders the template and writes the result to a file

### 3.4. Fix Functions Implementation

1. Implement wrappers for the following functions:
   - `name_fix(text)` - Sanitizes a name by replacing special characters with underscores
   - `path_fix(text)` - Applies name_fix to the filename part of a path

### 3.5. Dedent Functions Implementation

1. Implement wrappers for the following functions:
   - `dedent(text)` - Removes common leading whitespace from a multiline string
   - `prefix(text, prefix)` - Adds a prefix to each line of a multiline string

### 3.6. Error Handling

1. Implement helper functions to convert Rust errors to Rhai errors:
   - `io_error_to_rhai_error` - Converts io::Error to EvalAltResult
   - `tera_error_to_rhai_error` - Converts tera::Error to EvalAltResult
   - `string_error_to_rhai_error` - Converts String error to EvalAltResult

## 4. Example Usage in Rhai Scripts

### TextReplacer Example
```rhai
// Create a TextReplacer with multiple replacements
let replacer = text_replacer_new()
    .pattern("foo").replacement("bar").regex(true)
    .pattern("hello").replacement("world")
    .build();

// Use the replacer
let result = replacer.replace("foo hello foo");
println(result);  // Outputs: "bar world bar"

// Replace in a file
let file_result = replacer.replace_file("input.txt");
println(file_result);

// Replace and write to a new file
replacer.replace_file_to("input.txt", "output.txt");
```

### TemplateBuilder Example
```rhai
// Create a TemplateBuilder
let template = template_builder_open("template.txt")
    .add_var("name", "John")
    .add_var("age", 30)
    .add_var("items", ["apple", "banana", "cherry"]);

// Render the template
let result = template.render();
println(result);

// Render to a file
template.render_to_file("output.html");
```

### Fix and Dedent Examples
```rhai
// Use name_fix and path_fix
let fixed_name = name_fix("Hello World!");
println(fixed_name);  // Outputs: "hello_world"

let fixed_path = path_fix("/path/to/Hello World!");
println(fixed_path);  // Outputs: "/path/to/hello_world"

// Use dedent and prefix
let indented_text = "    line 1\n    line 2\n        line 3";
let dedented = dedent(indented_text);
println(dedented);  // Outputs: "line 1\nline 2\n    line 3"

let text = "line 1\nline 2\nline 3";
let prefixed = prefix(text, "    ");
println(prefixed);  // Outputs: "    line 1\n    line 2\n    line 3"